function [u_A, u_E, u_P] = ACX_CalcServoCommands( swash_cmd, ...
                                                  swash_cfg, ...
                                                  servo_cfgs )
%##########################################################################
% [u_servo] = ACX_CalcServoCommands( swash_cmd, swash_cfg, servo_cfgs )
%
% Calculate the servo commands based on swashplate pose.
% We use a linearized version of the swashplate inverse kinematics to 
% calculate the servo angles. Subsequently we use a servo black-box model
% to convert the servo angles to a value between 0 and 1.
% 1: maximum position (arm up)
% 0: minimum position (arm down)
%
% u_E        : "Elevator" command. u_E eof (-1,1).
% u_A        : "Aileron" command.  u_A eof (-1,1).
% u_P        : "Pitch" command.    u_P eof (-1,1).
%
%
% swash_cmd  :  Swashplate pose swash_cmd = [phi_1c, phi_1s, h_sp] in SI-
%               units -> [rad], [rad], [m].
% swash_cfg  :  Parameters defining the swashplate inverse kinematics.
% servo_cfgs :  Servo model parameters.
%--------------------------------------------------------------------------
% Author: Christoph Huerzeler (ASL-ETHZ)
% Date: 2011
% Email: christoph.huerzeler@mavt.ethz.ch
%##########################################################################
%#codegen

%% Calculate the servo commands from desired swashplate pose
phi_1c = swash_cmd(1);
phi_1s = swash_cmd(2);
h_sp   = swash_cmd(3);

%% Calculate the desired servo angles
kE    = swash_cfg.ElevatorCoeffs;
kA    = swash_cfg.AileronCoeffs;
kP    = swash_cfg.PitchCoeffs;
phi_E = kE(1)*phi_1c + kE(3)*h_sp   + kE(4)/h_sp;
phi_A = kA(1)*phi_1c + kA(2)*phi_1s + kA(3)*h_sp + kA(4)/h_sp;
phi_P = kP(1)*phi_1c + kP(2)*phi_1s + kP(3)*h_sp + kP(4)/h_sp;


%% Calculate the desired servo signals
% Elevator
servo_cfg = servo_cfgs(1);
k1        = servo_cfg.StaticCoeffs(1);
k2        = servo_cfg.StaticCoeffs(2);
k3        = servo_cfg.StaticCoeffs(3);
u_E = (-k2 + sqrt(k2^2 - 4*k1*(k3-phi_E)))/(2*k1);

% Aileron
servo_cfg = servo_cfgs(2);
k1        = servo_cfg.StaticCoeffs(1);
k2        = servo_cfg.StaticCoeffs(2);
k3        = servo_cfg.StaticCoeffs(3);
u_A = (-k2 + sqrt(k2^2 - 4*k1*(k3-phi_A)))/(2*k1);

% Pitch
servo_cfg = servo_cfgs(3);
k1        = servo_cfg.StaticCoeffs(1);
k2        = servo_cfg.StaticCoeffs(2);
k3        = servo_cfg.StaticCoeffs(3);
u_P = (-k2 + sqrt(k2^2 - 4*k1*(k3-phi_P)))/(2*k1);

%% --- end of file