function [phi_1c, phi_1s, h_sp] = ACX_CalcSwashplatePoseV2( u_ctrl,    ...
                                                            phase_offset, ...
                                                          rotor_cfg, ... 
                                                          swash_cfg, ...
                                                          h_sp_offset )                                           
%##########################################################################
% [phi_1c, phi_1s] = ACX_CalcSwashplatePose( u_ctrl, rotor_cfg, swash_cfg )
%
% Converts pitch and roll commands into desired swashplate pose.
% We assume the rotor rotates in CW direction and maximum cyclic follows
% with 90 delay to maximum swashplate tilt-up.
%
% phi_1c       : Longitudinal swashplate angle. [rad]
% phi_1s       : Lateral swashplate angle.      [rad]
% h_sp         : Swashplate height.             [m]
%
%
% u_pitch      : The calculated pitch control action. u_pitch eof (-1,1).
% u_roll       : The calculated roll control action.  u_roll  eof (-1,1).
% u_yawNthrust : The calculated yaw and thrust control action. 
%                the correct mixing of yaw and thrust is expected to 
%                happen before this function.
%                u_yawNthrust eof (-1,1).
% rotor_cfg    : Struct storing the important rotor parameters.
% swash_cfg    : Struct storing the important swashplate parameters.
%
%--------------------------------------------------------------------------
% Author: Christoph Huerzeler (ASL-ETHZ)
% Date: 2011
% Email: christoph.huerzeler@mavt.ethz.ch
%##########################################################################
%#eml

%phi_max  = deg2rad( 10 );
%h_sp_min = 19.7/1000;
%h_sp_max = 29.7/1000;
%% Extract configuration
phi_max  = swash_cfg.MAX_ANGLE;
h_sp_min = swash_cfg.MIN_HEIGHT; % 21/1000; % swash_cfg.MIN_HEIGHT;  !!!
h_sp_max = swash_cfg.MAX_HEIGHT;
phi_rot  = rotor_cfg.FLAP_LAG+phase_offset;

%% Extract control actions
u_roll       = u_ctrl(1);
u_pitch      = u_ctrl(2);
u_yawNthrust = u_ctrl(3);

%% Account for swashplate to cyclic and cyclic to flapping phase delays
u_1c =  u_pitch * cos(phi_rot - pi/2) - u_roll * sin(phi_rot - pi/2);
u_1s = -u_pitch * sin(phi_rot - pi/2) - u_roll * cos(phi_rot - pi/2);

%% Convert the pitch-roll control commands to swashplate angles
phi_1c = phi_max * u_1c;
phi_1s = phi_max * u_1s;

%% Convert the yaw-thrust control command to swashplate height
% h_sp_max = max swashplate height
% h_sp_min = min swashplate height
% h_amp = active middle point = (h_sp_max + h_sp_min)/2 + offset; 
% Switching Linear approximation (problem with the derivative):
% If u_yaw>0:  h_sp_out=h_amp+(h_sp_max-h_amp)*u_yaw;
% If u_yaw<=0: h_sp_out=h_amp+(h_amp-h_sp_min)*u_yaw;
h_amp = h_sp_offset + (h_sp_max + h_sp_min)/2; 
if(u_yawNthrust<=0)
    h_sp=h_amp+(h_amp-h_sp_min)*u_yawNthrust;
else
    h_sp=h_amp+(h_sp_max-h_amp)*u_yawNthrust;
end
%h_sp = (h_sp_max - h_sp_min) * u_yawNthrust + h_sp_min;

%% --- end of file
