function SSTest = ACX_SIM_Check_SS_Stab_Ctrb_Obs(ssmodel)
%
%   Syntax:
%   SSTest = ACX_Check_SS_Stab_Ctrb_Obs(ssmodel)
%   
%   ### AIROBOTS COAX Small System Identification Toolbox
%
%   Inputs:
%   ssmodel     : A state space model (or ss id_model)
%
%   Outputs:
%   SSTest      : Structure that contains all stability information ->
%       SSTest.eigen.values     : the eigenvalues of matrix A
%       SSTest.eigen.V          : "nobalance" matrix of eigenvectors
%       SSTest.eigen.D          : "nobalance" matrix of eigenvalues
%       SSTest.singular.S       : Singular values of A
%       SSTest.singular.U and  
%       SSTest.singular.V       : unitary matrices such tat A = U*S*V'
%       SSTest.ctrb.FullyControllable : == 1 if fully controllable
%       SSTest.ctrb.UncontrollableStates : number of uncontrollable states
%       SSTest.ctrb.CtrbMatrix  : Controllability Matrix
%       SSTest.obsv.FullyObservable : == 1 if fully observable
%       SSTest.obsv.UnobservableStates : number of unobservable states
%       SSTest.obsv.ObservabilityMatrix: observability matrix
%
%   This function will also print out a red message if positive eigenvalues
%   are found (unstable A) and a black message if critically stable poles
%   are found.
%
%   Authors : Kostas Alexis (konstantinos.alexis@mavt.ethz.ch)
%
Amatrix = ssmodel.A;
Bmatrix = ssmodel.B;
Cmatrix = ssmodel.C;
% Dmatrix = ssmodel.D;

% compute eigenvalues

eigsA = eig(Amatrix);

flag_unstable = 0;
flag_critical = 0;
for i = 1:length(eigsA)
    if(eigsA(i)>0)
        flag_unstable = 1;
    end
    if(eigsA(i) == 0)
        flag_critical = 1;
    end
end

if(flag_unstable+flag_critical>0)
    if(flag_unstable==1)
        SSTest.StabFlag = 0;
        cprintf('SystemCommands','*****************************************\n');
        cprintf('-Red',          'THE SYSTEM HAS UNSTABLE POLES ! ! ! ! ! !\n');
        cprintf('SystemCommands','*****************************************\n');
    else
        SSTest.StabFlag = 0;
        cprintf('SystemCommands','*****************************************\n');
        cprintf('Black',         'THE SYSTEM HAS CRITCALLY STABLE POLES ! !\n');
        cprintf('SystemCommands','*****************************************\n');
    end
else
    SSTest.StabFlag = 1;% SYSTEM IS STABLE
end


% no balancing eigenvalues
[V,D] = eig(Amatrix,'nobalance');
[U,S,Vs] = svd(Amatrix);

SSTest.eigen.values = eigsA;
SSTest.eigen.V = V;
SSTest.eigen.D = D;

SSTest.singular.S = S;
SSTest.singular.U = U;
SSTest.singular.V = Vs;

[rA,cA] = size(Amatrix);
[rB,cB] = size(Bmatrix);

% generalized eigenvalues (if A,B square)
if((rA==cA)&&(rA==rB)&&(rB==cB))
    StabTest.genEigen  = eig(Amatrix,Bmatrix);
else
    StabTest.genEigen = [];
end

Co = ctrb(Amatrix,Bmatrix);
unco = length(Amatrix) - rank(Co);

if(unco~=0)
    SSTest.ctrb.FullyControllable = 0;
else
    SSTest.ctrb.FullyControllable = 1;
end

SSTest.ctrb.UncontrollableStates = unco;
SSTest.ctrb.CtrbMatrix = Co;

Ob = obsv(Amatrix,Cmatrix);
unob = length(Amatrix)-rank(Ob);

if(unob~=0)
    SSTest.obsv.FullyObservable = 0;
else
    SSTest.obsv.FullyObservable = 1;
end

SSTest.obsv.UnobservableStates = unob;
SSTest.obsv.ObsMatrix = Ob;


end
    
    