%%  ACX ID MISOA
%
%   ACX Multi-Input/Single-Output Analysis
%   
%   This Script provides all the required functionality to perform
%   Multiple-Input/Single Output Analysis. It is supposed to aim the user
%   to understand and validate the relationships between inputs and
%   outputs. Moreover, this script provides all the required metrics and
%   the frequency plots to directly perform first calculations about
%   input/output relationships. 
%
%   FOR HELP type: "help ACX_ID_MULTI_INPUT_ID"
%   
%   Authors: Kostas Alexis (alexisk@ethz.ch)

clc
cprintf('Blue','### ACX ID MISOA Tool Initialized!\n'); 
cprintf('Green', '### Please load the required Dataset\n');
cnt_mrg = 1;

%%  ACX MISOA: Merge Experiments (requires specific structure of the datasets)
% 
% InputsArray_A = InputsArray;
% OutputsArray_A = OutputsArray;
% clear InputsArray; clear OutputsArray;
% 
% [InputsArray,OutputsArray] = ACX_SIM_MergeExperiments(InputsArray_A,OutputsArray_A,InputsArray,OutputsArray);
% 

%%  ACX MISOA: User's Commands Structure (Fill the Flags)

SignalPreparationFlags.ManualInterpretation.Flag = 1;
SignalPreparationFlags.ManualInterpretation.TimePlot = 1;
SignalPreparationFlags.ManualInterpretation.ffPlot = 1;
SignalPreparationFlags.ManualInterpretation.BodePlot = 1;
SignalPreparationFlags.ManualInterpretation.TD2FDmethod = 'etfe'; % 'etfe' , 'spa', 'spafdr
SignalPreparationFlags.ManualInterpretation.FilterData = 1;
SignalPreparationFlags.ManualInterpretation.FilterDesigned = 0;
SignalPreparationFlags.BuildInputOutputIDobjects.Flag = 1;
SignalPreparationFlags.BuildInputOutputIDobjects.BuildIDFRDobjectsMethod = 'etfe'; % 'etfe, 'spa', 'spafdr'
SignalPreparationFlags.BuildInputOutputIDobjects.AdviceMe = 1;
SignalPreparationFlags.BuildInputOutputIDobjects.DelayEst = 1;
SignalPreparationFlags.BuildInputOutputIDobjects.BodePlot = 1;
SignalPreparationFlags.BuildInputOutputIDobjects.Pexcit = 1; 
SignalPreparationFlags.BuildInputOutputIDobjects.CheckCoherence = 1;
SignalPreparationFlags.CreateMIMOIDDOBJS = 1;
SignalPreparationFlags.DetrendOutputs = 1;
SignalPreparationFlags.DetrendInputs = 1; 
SignalPreparationFlags.FilterOutputs = 1;
SignalPreparationFlags.FilterInputs = 1;
SignalPreparationFlags.CreateMIMOIDFRDmethod = 'spa'; % 'etfe', 'spa', 'spafdr'

%%  Create Input/Output Matrices

time_vec = Ts*(1:length(ID_Datasets.OpenLoop{1}.p.values)) - Ts;
OutputsArray(:,1) = time_vec;
OutputsArray(:,2) = ID_Datasets.OpenLoop{1}.p.values;
InputsArray(:,1) = time_vec;
InputsArray(:,2) = ID_Datasets.OpenLoop{1}.phi1c.values;
InputsArray(:,3) = ID_Datasets.OpenLoop{1}.phi1s.values;

%%  ACX ID MISOA: Options Structure
ExperimentLength = length(time_vec);
sample_time = Ts;
Slowest_Oscillation = 4; % seconds
T_win = 5*Slowest_Oscillation;
Window_Length = floor(T_win/sample_time);
MaxExperimentLength = max(ExperimentLength);
if(Window_Length>(MaxExperimentLength/2))
    cprintf('-Red','Your Window Length must be less than 0.5*T_rec\n');
end
%Window_Length = 1024;

MISOoptions.Window = hanning(Window_Length);
MISOoptions.WindowLen = Window_Length;
MISOoptions.Noverlap = floor(Window_Length/2);
MISOoptions.NFFT = 1024*8;
MISOoptions.noverlap = MISOoptions.Noverlap;
MISOoptions.nfft = MISOoptions.NFFT;
MISOoptions.plot = 1;
MISOoptions.f1 = rad_s2Hz(0.01); % 0.01592; rad_s2Hz(0.01);  %  for pitch chirp - z I found good f1 = rad_s2Hz(0.01),
MISOoptions.f2 = rad_s2Hz(6.28); %2;
MISOoptions.m = 1024;
MISOoptions.ChirpZ = 1;
MISOoptions.detrend = 1;
MISOoptions.Fs = 1/sample_time;
MISOoptions.Ts = Ts;
MISOoptions.C_e = sqrt(0.55); 


%%  ACX ID MISOA: Coherences and Frequency ratio Analysis

OutputNumber = 1; 

[coherences,coherencesMeans] = SIM_MISO_FreqAnalysis(OutputsArray,InputsArray,SignalPreparationFlags,sample_time,OutputNumber,MISOoptions);

%%  ACX ID MISOA: Spectrogram Between Input and Output

OutputNumber = 1;
InputNumber = 1;

SpectroInfo = SIM_SpectrogramIO(InputsArray(:,InputNumber+1),OutputsArray(:,OutputNumber+1),MISOoptions);

%%  ACX ID MISOA: Power Spectral Estimate

OutputNumber = 1;
uu = InputsArray(:,2:end);
yy = OutputsArray(:,OutputNumber+1);

[PSDuu,PSDyy,PSDuy] = SIM_PSD_MISO(uu,yy,MISOoptions);
close all
%%  ACX ID MISOA: Coherences between Inputs (check the role of primary and secondary inputs)

[InputCrossCoherences,InputCrossCoherencesMeans] = SIM_MISO_InputCrossCoherence(InputsArray,sample_time,MISOoptions);

%%  ACX ID MISOA: 2 Inputs - 1 Output MISO State-Space ID
%
%   In rotorcrafts it is very common to have one primary input that is
%   supposed to control one specific degree of freedom and a secondary
%   input that also excites the same degree of freedom. Having this in midn
%   we automate the process of a typical 2 Inputs - 1 Output MISO
%   State-Space model identification.

clear out; clear inps;
%   Specify Output
output_i = 1;

%   Specify the two inputs
input_i_1 = 1;
input_i_2 =2;

%   Build 2 Inputs-1 Output MISO system
out = detrend(OutputsArray(:,output_i+1));
inps(:,1) = detrend(InputsArray(:,input_i_1+1));
inps(:,2) = detrend(InputsArray(:,input_i_2+1));

%   Build Time-Domain IDDATA object

data = iddata(out,inps,Ts);
data = detrend(data);

%   Specify initial parameters
par_init = [-1 -1 1 1];
%par_init = TWO_I_ONE_O_SSIDGREY.par; par_init = par_init';
%   Specify initial state for y (not x_1 and x_2)

init_state = 0;

cprintf('SystemCommands', '********************************************************************\n');
cprintf('-Blue',          'Note that sometimes TD lead to different ID compared to FD data!\n');
cprintf('-Red',           'This seems to be related with the stability properties of the system\n');
cprintf('SystemCommands', '********************************************************************\n');


%   Select Data Type [0-6] (TD,FD,ETFE,SPA,SPAFDR,CZTidfrd)
data_type = 1;
ok_flag = 0;
switch data_type
    case 1
        two_i_one_o_data = data;
        cprintf('-Blue','TIME-DOMAIN Data object\n');
        ok_flag = 1;
    case 2
        two_i_one_o_data = fft(data);
        cprintf('-Blue','FREQ-DOMAIN FFT Data object\n');
        ok_flag = 1;
    case 3 
        two_i_one_o_data = etfe(data);
        cprintf('-Blue','ETFE ESTIMATION object\n');
        ok_flag = 1;
    case 4
        two_i_one_o_data = spa(data,MISOoptions.WindowLen,ww);
        cprintf('-Blue','SPA ESTIMATION object\n');
        ok_flag = 1;
    case 5
        two_i_one_o_data = spafdr(data);
        cprintf('-Blue','SPAFDR ESTIMATION object\n');
        ok_flag = 1;
    case 6
        two_i_one_o_data = CZTidfrd(data,MISOoptions);
        cprintf('-Blue','FREQ-DOMAIN CZT Data object\n');
        cprintf('-SystemCommands','Chirp-Z is extremely sensitive in the correct frequency selection!\n');
        ok_flag = 1;
    otherwise
        cprintf('-Red','You must select between 1->6 \n');
end


if(ok_flag == 1)
    TWO_I_ONE_O_SSIDGREY = ACX_SIM_TWO_I_ONE_O_SS(two_i_one_o_data,par_init,init_state,MISOoptions)
    TWO_I_ONE_O_SS = ss(TWO_I_ONE_O_SSIDGREY.A,TWO_I_ONE_O_SSIDGREY.B,TWO_I_ONE_O_SSIDGREY.C,TWO_I_ONE_O_SSIDGREY.D);
end

SSTest = ACX_SIM_Check_SS_Stab_Ctrb_Obs(TWO_I_ONE_O_SS);

id_metrics_flag = input('Do you want to see ID metrics (based on the same dataset here)[0/1]');
if(id_metrics_flag == 1)
    MdlMetrics = ACX_SIM_MATLABSID_METRICS(TWO_I_ONE_O_SSIDGREY,data)
    cprintf('-Blue','Note that you should make the validation also with a data set not used in ID\n');
end

%%  ACX ID MISOA: 2 Inputs - 1 Output 3rd Order MISO State-Space ID
%
%   In rotorcrafts it is very common to have one primary input that is
%   supposed to control one specific degree of freedom and a secondary
%   input that also excites the same degree of freedom. Having this in midn
%   we automate the process of a typical 2 Inputs - 1 Output MISO
%   State-Space model identification. In addition to the above we thought
%   that it may makes some sense to provide also the capability to model it
%   as a third order system...

clear out; clear inps;
%   Specify Output
output_i = 1;

%   Specify the two inputs
input_i_1 = 1;
input_i_2 =2;

%   Build 2 Inputs-1 Output MISO system
out = detrend(OutputsArray(:,output_i+1));
inps(:,1) = detrend(InputsArray(:,input_i_1+1));
inps(:,2) = detrend(InputsArray(:,input_i_2+1));

%   Build Time-Domain IDDATA object

data = iddata(out,inps,Ts);
data = detrend(data);

%   Specify initial parameters
par_init = [-1 -1 -1 1 1];

%   Specify initial state for y (not x_1 and x_2)

init_state = 0;

cprintf('SystemCommands', '********************************************************************\n');
cprintf('-Blue',          'Note that sometimes TD lead to different ID compared to FD data!\n');
cprintf('-Red',           'This seems to be related with the stability properties of the system\n');
cprintf('SystemCommands', '********************************************************************\n');


%   Select Data Type [0-6] (TD,FD,ETFE,SPA,SPAFDR,CZTidfrd)
data_type = 1;
ok_flag = 0;
switch data_type
    case 1
        two_i_one_o_data = data;
        cprintf('-Blue','TIME-DOMAIN Data object\n');
        ok_flag = 1;
    case 2
        two_i_one_o_data = fft(data);
        cprintf('-Blue','FREQ-DOMAIN FFT Data object\n');
        ok_flag = 1;
    case 3 
        two_i_one_o_data = etfe(data);
        cprintf('-Blue','ETFE ESTIMATION object\n');
        ok_flag = 1;
    case 4
        two_i_one_o_data = spa(data,MISOoptions.WindowLen,ww);
        cprintf('-Blue','SPA ESTIMATION object\n');
        ok_flag = 1;
    case 5
        two_i_one_o_data = spafdr(data);
        cprintf('-Blue','SPAFDR ESTIMATION object\n');
        ok_flag = 1;
    case 6
        two_i_one_o_data = CZTidfrd(data,MISOoptions);
        cprintf('-Blue','FREQ-DOMAIN CZT Data object\n');
        cprintf('-SystemCommands','Chirp-Z is extremely sensitive in the correct frequency selection!\n');
        ok_flag = 1;
    otherwise
        cprintf('-Red','You must select between 1->6 \n');
end


if(ok_flag == 1)
    TWO_I_ONE_O_3ORDER_SSIDGREY = ACX_SIM_TWO_I_ONE_O_3ORDER_SS(two_i_one_o_data,par_init,init_state,MISOoptions)
    TWO_I_ONE_O_3ORDER_SS = ss(TWO_I_ONE_O_3ORDER_SSIDGREY.A,TWO_I_ONE_O_3ORDER_SSIDGREY.B,TWO_I_ONE_O_3ORDER_SSIDGREY.C,TWO_I_ONE_O_3ORDER_SSIDGREY.D);
end

SSTest = ACX_SIM_Check_SS_Stab_Ctrb_Obs(TWO_I_ONE_O_3ORDER_SS);

id_metrics_flag = input('Do you want to see ID metrics (based on the same dataset here)[0/1]');
if(id_metrics_flag == 1)
    MdlMetrics = ACX_SIM_MATLABSID_METRICS(TWO_I_ONE_O_3ORDER_SSIDGREY,data)
    cprintf('-Blue','Note that you should make the validation also with a data set not used in ID\n');
end
 
