function SISO_TF = ACX_SIM_ID_Process_Model_ID(data,SISO_TF,options)
%
%   Syntax:
%   SISO_TF = ACX_SIM_ID_Process_Model_ID(data,TypeString)
%
%   SISO Process Model ID. This function is just an abstraction of MATLAB's
%   Process Model ID. It was built just to unify the way functions are
%   called within ACX_ID_TOOL. However note that typical IDPROC-PEM has
%   more capabilities. 
%
%   Note that this function CAN BE CALLED with CHIRP-Z TRANSFORM BASED
%   frequency responses (added by the ACX_ID_TOOLBOX).
%
%   Note that this function can be called having SISO_TF pre-defined as
%   idproc (type "help idproc") so one can pre-fix a subset of the values
%   and parameters to be identified.
%
%
%   Inputs:
%   data        : the data object (Time Domain, FFT, Chirp-Z) over which
%   the process model will be estimated
%   SISO_TF     : initialized using idproc PROCESS MODEL
%                 SISO_TF must have been initialized using a TypeString
%                 The TypeString must/may contain the following:
%               P for Process Model (required)
%               0,1,2 or 3 for process poles (required)
%               D to incluse time delay (optional)
%               Z to include a process zero (optional)
%               U to indicate possible complex-valued poles (optional)
%               I to indicate enforced integration (optional)
%   options     : structure that contain the following options:
%       options.dist = 0 -> no disturbance model
%                      1 -> 1st order arma model
%                      2 -> 2nd order arma model
%       
%
%   Outputs:
%   SISO_TF     : SISO Process Model Identified Transfer Function
%
%   Authors: Kostas Alexis (konstantinos.alexis@mavt.ethz.ch)
%

if(isfield(options,'dist'))
    if(options.dist == 1)
        SISO_TF = pem(data,SISO_TF,'dist','arma1');
    end
    if(options.dist == 2)
        SISO_TF = pem(data,SISO_TF,'dist','arma2');
    end
    if(options.dist == 0)
        SISO_TF = pem(data,SISO_TF);
    end
else
    SISO_TF = pem(data,SISO_TF);
end
    % SISO_TF = CoherePem(data,SISO_TF);

[mag,phase,wout] = bode(SISO_TF);

for i = 1:length(mag(1,1,:));
    magi(i) = mag(1,1,i);
    phasei(i) = phase(1,1,i);
end

[yout,tout] = step(SISO_TF);

figure
subplot(2,2,[1 3])
plot(tout,yout)
xlim([tout(1) tout(end)]);
xlabel('Time (s)'); ylabel('Step Response');
grid on
subplot(2,2,2)
semilogx(wout,20*log10(abs(magi)));
xlabel('Frequency (rad/s)'); ylabel('Magnitude (dB)');
grid on
subplot(2,2,4)
semilogx(wout,phasei)
grid on
xlabel('Frequency (rad/s)'); ylabel('Phase (degrees)');

end