%%  ACX ID SISOA
%
%   ACX Single-Input/Single-Output Analysis and Identification
%   
%   This Script provides all the required functionality to perform
%   Single-Input/Single Output Analysis and Identification. 
%
%   Provided Functionality:
%   1.  Time Plots
%   2.  ID Advice
%   3.  Data Detrending
%   4.  Data Filtering
%   5.  Coherence & Random Error Checks
%   6.  Input/Output Spectrograms
%   7.  Empirical, spectral analysis with and without freq zoom TF estimate
%   8.  Chirp-Z Transform based measurements based TF estimate
%   9.  Process Model TF Identification based on Time-Domain
%   data, FFT based frequency responses, Chirp-Z based frequency responses
%   10. ARX Black Box Models
%   11. ARMAX Black Box Models
%   12. State-Space based 1st Order State Space & TF ID
%   13. Rough (FFT ratio) TF ID
%   14. TF ID based om Transfer Function Determination Code with special
%   version for Coherence - weighted ID optimization
%
%   For theoretical help type "doc ACX_ID_TRANSFER_FUNCTION_ID"
%   
%   Authors: Kostas Alexis (konstantinos.alexis@mavt.ethz.ch)

clc
cprintf('Blue', '### ACX ID SISOA: Single Input-Single Output Analysis & ID\n');
%%  DEFINE input_vec and output_vec (column vectors)


%%  SISO Options Structure
Fs = 1/Ts;
sample_time = Ts;

Slowest_Oscillation = 10.5; % 10; % seconds
T_win = 2*Slowest_Oscillation;
Window_Length = floor(T_win/sample_time);

SISOoptions.FDmethod = 'etfe';
SISOoptions.N = 128;
SISOoptions.auto = 1;
SISOoptions.Fs = 1/Ts;
SISOoptions.f1 = rad_s2Hz(0.01);  % in RAD/S % for pitch chirp - z I found good f1 = rad_s2Hz(0.01),
SISOoptions.f2 = rad_s2Hz(6.28); % 12 - 15 ? in RAD/S % for pitch chirp - z I found good f2 = rad_s2Hz(8.28)
SISOoptions.a = exp(1i*2*pi*SISOoptions.f1/Fs);
SISOoptions.Ts = Ts; 
SISOoptions.Window = hanning(Window_Length); %1024
SISOoptions.Noverlap = floor(Window_Length/2); % 512
SISOoptions.NFFT = 1024*8; % 1024
SISOoptions.plot = 1;
SISOoptions.C_e = sqrt(0.55); 
SISOoptions.WindowLen = Window_Length;
SISOoptions.Resol = SISOoptions.NFFT;
SISOoptions.m = 1024;


%%  ACX SISOA: TD Data Object Build 

data = iddata(output_vec,input_vec,Ts);% data = detrend(data);

%%  ACX SISOA: Time Plot

time_vec = (1:length(output_vec))*Ts; time_vec = time_vec - Ts;

figure
subplot(2,1,1)
plot(time_vec,output_vec);
xlabel('Time (s)'); ylabel('Output');
subplot(2,1,2)
plot(time_vec,input_vec);
xlabel('Time (s)'); ylabel('Input');

%%  ACX SISOA: PASS SIGNALS THROUGH WINDOWS (input-output vectors)

input_vec = SIM_ConvWindow(input_vec,SISOoptions.Window);
output_vec = SIM_ConvWindow(output_vec,SISOoptions.Window);

%%  ACX SISOA: PASS SIGNALS THROUGH WINDOWS (iddata object)

data.InputData = SIM_ConvWindow(data.InputData,SISOoptions.Window);
data.OutputData = SIM_ConvWindow(data.OutputData,SISOoptions.Window);

%%  ACX SISOA: Time Plot for (input,output vectors) and (iddata vectors)

time_vec = (1:length(output_vec))*Ts; time_vec = time_vec - Ts;

figure
subplot(2,1,1)
plot(time_vec,output_vec);
hold on
plot(time_vec,data.OutputData,'r');
xlabel('Time (s)'); ylabel('Output');
subplot(2,1,2)
plot(time_vec,input_vec);
hold on
plot(time_vec,data.InputData,'r');
xlabel('Time (s)'); ylabel('Input');

%%  ACX SISOA: MATLAB Advice

advice(data)

%%  ACX SISOA: Get Trend Info

getTrend(data)

%%  ACX SISOA: Possible Data Detrending

DetrendFlag = 1; % CHOOSE HERE IF YOU WANT TO DETREND!

if(DetrendFlag == 1)
    data = detrend(data);
end

%%  ACX SISOA: SISO Coherence Checks

[NomChecks,FAChecks] = ACX_SIM_Check_SISO_IO_FIXCOHTHRES(output_vec,input_vec,Ts,SISOoptions);
FAChecks.Coherence.meanValue

%%  ACX SISOA: Input/Output Spectrogram

[Spectro] = SIM_SpectrogramIO(input_vec,output_vec,SISOoptions);

%%  ACX SISOA: Input/Output Power Spectral Density Estimate

[PSDuu,PSDyy,PSDuy] = SIM_PSD_IO(input_vec,output_vec,SISOoptions); 

%%  ACX SISOA: Input/Output Delay Estimation

EstimatedDelayInSamples = delayest(data)

%%  ACX SISOA: Chirp-Z Transform Based SISO ID

CZT_idfrd = SIM_CZTidfrd(data,SISOoptions);


out_czt = CZT_idfrd.OutputData;
inp_czt = CZT_idfrd.InputData;

IOratio(1:length(out_czt))=0;
for i =1:length(out_czt)
IOratio(i) = out_czt(i)/inp_czt(i);
end

mag = 20*log10(abs(IOratio));

phase = angle(IOratio);

wwlin = linspace(Hz2rad_s(SISOoptions.f1),Hz2rad_s(SISOoptions.f2),length(mag));
figure;
subplot(2,1,1)
semilogx(wwlin,mag);
subplot(2,1,2)
semilogx(wwlin,phase/pi*180);

cprintf('Blue','Chirp-Z Transform based frequency responses object built\n');

%%  ACX SISOA: Process Model ID
%   This functionality provides the capability to identify a SISO transfer
%   function. It is based on "Process Models ID" since we expect that SISO
%   models will be of relatively low order. At least this makes sense in
%   terms of physical interpetation. 

%   Type String:
%   P for Process Model (required)
%   0,1,2 or 3 for process poles (required)
%   D to incluse time delay (optional)
%   Z to include a process zero (optional)
%   U to indicate possible complex-valued (underdamped) poles (optional)
%   I to indicate enforced integration (optional)
%
%   You can make some of the parameters 'fixed' if you know them from
%   previous data. Just type "help idproc" to find out how!

cprintf('SystemCommands', '********************************************************************\n');
cprintf('-Blue',          'Note that sometimes TD lead to different ID compared to FD data!\n');
cprintf('-Red',           'This seems to be related with the stability properties of the system\n');
cprintf('SystemCommands', '********************************************************************\n');

%SISOoptions.f2 = rad_s2Hz(6.58)
clear PM_TypeString;
PM_TypeString = 'P3Z';
SISO_TF = idproc(PM_TypeString);

%   Select Data Type [0-6] (TD,FD,ETFE,SPA,SPAFDR,CZTidfrd)
data_type = 2;
ok_flag = 0;
switch data_type
    case 1
        proc_id_data = data;
        cprintf('-Blue','TIME-DOMAIN Data object\n');
        ok_flag = 1;
    case 2
        proc_id_data = fft(data);
        cprintf('-Blue','FREQ-DOMAIN FFT Data object\n');
        ok_flag = 1;
    case 3 
        proc_id_data = etfe(data);
        cprintf('-Blue','ETFE ESTIMATION object\n');
        ok_flag = 1;
    case 4
        proc_id_data = spa(data);
        cprintf('-Blue','SPA ESTIMATION object\n');
        ok_flag = 1;
    case 5
        proc_id_data = spafdr(data);
        cprintf('-Blue','SPAFDR ESTIMATION object\n');
        ok_flag = 1;
    case 6
        proc_id_data = SIM_CZTidfrd(data,SISOoptions);
        cprintf('-Blue','FREQ-DOMAIN CZT Data object\n');
        cprintf('-SystemCommands','Chirp-Z is extremely sensitive in the correct frequency selection!\n');
        ok_flag = 1;
    otherwise
        cprintf('-Red','You must select between 1->6 \n');
end

cprintf('SystemCommands','*************************************************\n');
cprintf('Blue',          'Consider fixing the DC-GAIN of the IDed System!!!\n');
cprintf('SystemCommands','*************************************************\n');

if(ok_flag == 1)
    SISO_TF = ACX_SIM_ID_Process_Model_ID(proc_id_data,SISO_TF,SISOoptions);
    cprintf('Blue','SISO TF estimated!\n');
    cprintf('Blue','Note: In case you know some parameters already you can hardcode them and make the estimation status "fixed". Type "help idproc" to find out how.\n');
end

id_metrics_flag = input('Do you want to see ID metrics (based on the same dataset here)[0/1]');
if(id_metrics_flag == 1)
    MdlMetrics = ACX_SIM_MATLABSID_METRICS(SISO_TF,data);
     cprintf('-Blue','Note that you should make the validation also with a data set not used in ID\n');
end


%%  ACX SISOA: State-Space Based 2nd Order SISO Model ID

%ssdata = data;
ssdata = fft(data);
%ssdata = CZTidfrd(data,SISOoptions);

par_init = [-1 -1 1];
init_state = ssdata.OutputData(1); 

siso_ss_2nd_order = ACX_SIM_2ndOrder_SISO_SS_ID(ssdata,par_init,init_state,SISOoptions);
SSTest = ACX_Check_SS_Stab_Ctrb_Obs(siso_ss_2nd_order);

siso_ss_tf_2nd_order = ACX_SIM_SISO_2ndOrder_SS_TF_ID(ssdata,par_init,init_state,SISOoptions);

pole(siso_ss_tf_2nd_order)
dcgain(siso_ss_tf_2nd_order)
step(siso_ss_tf_2nd_order)

id_metrics_flag = input('Do you want to see ID metrics (based on the same dataset here)[0/1]');
if(id_metrics_flag == 1)
    MdlMetrics = ACX_SIM_MATLABSID_METRICS(siso_ss_2nd_order,data);
    cprintf('-Blue','Note that you should make the validation also with a data set not used in ID\n');
end

%%  ACX ID SISOA: State Space Model with 3 poles , 1 Zero
%
%   In rotorcrafts it is very common to have one primary input that is
%   supposed to control one specific degree of freedom and a secondary
%   input that also excites the same degree of freedom. Having this in midn
%   we automate the process of a typical 2 Inputs - 1 Output MISO
%   State-Space model identification. In addition to the above we thought
%   that it may makes some sense to provide also the capability to model it
%   as a third order system...

%   Specify number of poles, zeros [Currently n = 3 or 4, m = 1 or 2]
n = 3; % Number of Poles
m = 1; % Number of Zeros 
nm = [n m];
%   Specify initial parameters according to the n-m selection
par_init = [1 1 1 1 1];
%par_init = [80 146 81 16 1 1 ];
%   Specify initial state for y (not x_1 and x_2)

init_state = 0;

cprintf('SystemCommands', '********************************************************************\n');
cprintf('-Blue',          'Note that sometimes TD lead to different ID compared to FD data!\n');
cprintf('-Red',           'This seems to be related with the stability properties of the system\n');
cprintf('SystemCommands', '********************************************************************\n');


%   Select Data Type [0-6] (TD,FD,ETFE,SPA,SPAFDR,CZTidfrd)
data_type = 1;
ok_flag = 0;
switch data_type
    case 1
        two_i_one_o_data = data;
        cprintf('-Blue','TIME-DOMAIN Data object\n');
        ok_flag = 1;
    case 2
        two_i_one_o_data = fft(data);
        cprintf('-Blue','FREQ-DOMAIN FFT Data object\n');
        ok_flag = 1;
    case 3 
        two_i_one_o_data = etfe(data);
        cprintf('-Blue','ETFE ESTIMATION object\n');
        ok_flag = 1;
    case 4
        two_i_one_o_data = spa(data,MISOoptions.WindowLen,ww);
        cprintf('-Blue','SPA ESTIMATION object\n');
        ok_flag = 1;
    case 5
        two_i_one_o_data = spafdr(data);
        cprintf('-Blue','SPAFDR ESTIMATION object\n');
        ok_flag = 1;
    case 6
        two_i_one_o_data = CZTidfrd(data,SISOoptions);
        cprintf('-Blue','FREQ-DOMAIN CZT Data object\n');
        cprintf('-SystemCommands','Chirp-Z is extremely sensitive in the correct frequency selection!\n');
        ok_flag = 1;
    otherwise
        cprintf('-Red','You must select between 1->6 \n');
end

if(ok_flag == 1)
    SISO_SS_3PZGREY = ACX_SIM_PZ_SISO_SS_3PZ(two_i_one_o_data,par_init,init_state,SISOoptions,nm)
    SISO_SS_3PZ_TF =tf(SISO_SS_3PZGREY);
end

id_metrics_flag = input('Do you want to see ID metrics (based on the same dataset here)[0/1]');
if(id_metrics_flag == 1)
    MdlMetrics = ACX_SIM_MATLABSID_METRICS(SISO_SS_3PZGREY,data)
    cprintf('-Blue','Note that you should make the validation also with a data set not used in ID\n');
end
