function CZT_idfrd = SIM_CZTidfrd(td_iddata,options)
%
%   Syntax:
%   CZT_idfrd = SIM_CZTidfrd(td_iddata,options)
%
%   Inputs:
%   td_iddata   : System Identification Toolbox Time Domain IDDATA OBJECT.
%   options     : Options for Chirp-Z Transform.
%                 options.f1 : the Chirp-Z f1 Zoom Frequency (Hz)
%                 options.f2 : the Chirp-Z f2 Zoom Frequency (Hz)
%                 options.m  :
%                 options.a  :
%                 options.auto : if == 1 CZT option.m is handled automatic
%
%   Outputs:
%   CZT_idfrd   : Chirp-Z Transform based IDFRD object.
%
%   Function CZTidfrd builds a Chirp-Z Transform IDFRD object that can be 
%   used in Parameter Estimation Matlab Functions (pem) like typical IDFRD
%   objects of the System Identification Toolbox, which are computed using
%   FFT. Chirp-Z based analysis is supposed to be able to give better
%   results as long as you are able to know in which frequencies you should
%   focus.
%
%   Authors : Kostas Alexis (konstantinos.alexis@mavt.ethz.ch)
%

% if f1,f2 not defined:
% default: f1 = 0, f2 = Fs
if(isempty(td_iddata.Ts)~=1)
    Fs = 1/td_iddata.Ts;
    Ts = td_iddata.Ts;

    if(isfield(options,'f1')==0)
        options.f1 = 0;
    end

    if(isfield(options,'f2')==0)
        options.f2 = Fs;
    else
        if(options.f2>Fs)
            options.f2 = Fs;
            disp('f2 must be always <= Fs');
        end
    end

    if(isfield(options,'m')==0)
        options.m = length(td_iddata.OutputData)/2 + 1;
    end

    options.f1 = Hz2rad_s(options.f1); 
    options.f2 = Hz2rad_s(options.f2);
    w = exp(-1i*2*pi*(options.f2-options.f1)/(options.m*Fs));
    a = exp(1i*2*pi*options.f1/Fs);

    Y_czt = czt(td_iddata.OutputData,options.m,w,a);
    U_czt = czt(td_iddata.InputData,options.m,w,a);
%    Y_czt = czt(td_iddata.OutputData);
%    U_czt = czt(td_iddata.InputData);
    wwlin = linspace(options.f1,options.f2,length(Y_czt));
    CZT_idfrd = iddata(Y_czt,U_czt,Ts,'Frequency',wwlin);
    CZT_idfrd.Notes = 'Chirp-Z Transform based Frequency Response Data';

else
        disp('Error! You must specify Ts at the Time Domain Data Object');
end