function [coherences,coherencesMeans] = SIM_MISO_FreqAnalysis(OutputsArray,InputsArray,SignalPreparationFlags,sample_time,o_i,options)
%   
%   Syntax:
%   [coherences,coherencesMeans] = SIM_MISO_FreqAnalysis(OutputsArray,InputsArray,SignalPreparationFlags,sample_time,o_i,options)
%
%   Inputs:
%   OutputsArray    : Array with Time at the 1st column and all outputs at
%   the following columns
%   InputsArray     : Array with Time at the 1st column and all inputs at
%   the following columns
%   SignalPreparationFlags :Data structure with all the options of the
%   signal preparation process
%   sample_time     : sampling time
%   o_i             : output to examine
%   options         : Structure that contains the following:
%   options.Detrend : == 1 detrends the data
%   options.Window  : Windowing function and number of samples to
%                     use for each section
%   options.Noverlap: Number of samples by which the sections overlap
%   options.NFFT    : FT length which determines the frequencies at which 
%                     the coherence is estimated
%   options.ChirpZ  : ==1 if you want to do Chirp-Z based analysis
%   options.f1      : Lower Bound of Chirp-Z Frequency Zoom
%   options.f2      : Upper Bound of Chirp-Z Frequency Zoom
%   options.m       : Scalar that specifies the length of Chirp-Z Transform
%
%   Outputs:
%   coherences      : Coherences between all inputs and the specified
%   output
%   coherencesMeans : Mean values of the coherences between all inputs and
%   the specified output. 
%   
%   This Functions aims to aid the user to get some first understanding of
%   the system's input/output characteristics. It performs frequency
%   analysis for a specific output in relation with all the possible inputs
%   of the system.
%
%   Authors:    Kostas Alexis (konstantinos.alexis@mavt.ethz.ch)
%

options.dB = 1;
options.plot = 0;
[N_o,M_o] = size(OutputsArray);
M_o = M_o-1;
[N_i,M_i] = size(InputsArray);
M_i = M_i-1; 
Fs = 1/sample_time;

if(isfield(options,'detrend'))
    if(options.detrend == 1)
        OutputsArray_F = detrend(OutputsArray(:,o_i+1));
        for j = (1+1):(M_i+1)
            InputsArray_F(:,j) = detrend(InputsArray(:,j));
        end
    else
        OutputsArray_F = OutputsArray(:,o_i+1);
        for j = (1+1):(M_i+1)
            InputsArray_F(:,j) = InputsArray(:,j);
        end
    end
else
    OutputsArray_F = OutputsArray(:,o_i+1);
    for j = (1+1):(M_i+1)
        InputsArray_F(:,j) = InputsArray(:,j);
    end
end

for j = (1+1):(M_i+1)
	IDDATA_TD_OBJS{j-1} = iddata(OutputsArray_F,InputsArray_F(:,j),sample_time);
    IDDATA_FD_OBJS{j-1} = fft(IDDATA_TD_OBJS{j-1});
    
    [coherences(:,j-1), freqs(:,j-1)] = mscohere(InputsArray_F(:,j),OutputsArray_F,options.Window,options.Noverlap,options.NFFT,Fs);
    coherencesMeans(j-1) = sum(coherences(:,j-1))/length(coherences(:,j-1));
    if(options.ChirpZ==1)
        [magCZ(:,j-1),phaseCZ(:,j-1)] = SIM_Chirp_Z_FreqPlotSampledData(OutputsArray_F,InputsArray_F(:,j)',sample_time,options.f1,options.f2,options.m,options);
    end
    switch(SignalPreparationFlags.BuildInputOutputIDobjects.BuildIDFRDobjectsMethod)
    	case {'etfe'}
        	IDFRD_OBJS{j-1} = etfe(IDDATA_TD_OBJS{j-1});

        case {'spa'}
        	IDFRD_OBJS{j-1} = spa(IDDATA_TD_OBJS{j-1});
        case {'spafdr'}
        	IDFRD_OBJS{j-1} = spafdr(IDDATA_TD_OBJS{j-1});
            cprintf('Green','!!! spafdr has more capabilities when in the form: spafdr(iddata,R,w)\n');
        otherwise
        	cprintf('-Error','You must select either "etfe", "spa" or "spafdr"\n');
    end
	[mag(:,j-1),phase(:,j-1), wout(:,j-1)] = bode(IDFRD_OBJS{j-1});
                
end

figure; % [i*0.12 (i*1.5*0.1) (1-i*2*0.1)]  i*0.12
title('Multiple Input / Single Output Analysis');
for i = 1:M_i
    switch(i)
        case {1}
            color_string = 'b';
        case {2}
            color_string = 'r';
        case {3}
            color_string = 'g';
        case {4}
            color_string = 'k';
        case {5}
            color_string = 'm';
        case {6} 
            color_string = 'c';
        case {7}
            color_string = 'b--';
        case {8}
            color_string = 'r--';
        case {9} 
            color_string = 'g--';
        case {10}
            color_string = 'k--';
        case {11}
            color_string = 'm--';
        case {12}
            color_string = 'c--';
        otherwise
            color_string = 'y' % ok maybe i should make a colormapping function
    end
    subplot(2,2,1)
    title('Experimental BODE Magnitude Diagram');
    semilogx(wout(:,i),20*log10(mag(:,i)),color_string,'LineWidth',2);
    xlabel('Frequency (rad/s)'); ylabel('Amplitude (dB)');
    hold on
    grid on; box on;
    subplot(2,2,2)
    title('Experimental BODE Phase Diagram');
    semilogx(wout(:,i),phase(:,i),color_string,'LineWidth',2);
    xlabel('Frequency (rad/s)'); ylabel('Phase (degrees)');
    hold on
    grid on; box on; 
    subplot(2,2,3)
    title('Input/Output Coherences');
    hold on
    plot(freqs(:,j-i),coherences(:,i),color_string,'LineWidth',2)
    grid on; box on;
    xlabel('Frequency (Hz'); ylabel('Magnitude');
    if(options.ChirpZ==1)
        subplot(2,2,4)
        title('Chirp-Z based Experimental Magnitude Diagram');
        wwlin = linspace(options.f1,options.f2,length(magCZ(:,i)));
        semilogx(wwlin,magCZ(:,i),color_string,'LineWidth',2);
        hold on; grid on; box on;
        clear wwlin;
        xlabel('Frequency (Hz)'); ylabel('Chirp-Z based Output-Input Ratio Magnitude (dB)');
    end
    LegendNote{i} = ['C_{' num2str(i) ',' num2str(o_i) '} = ' num2str(coherencesMeans(i))];  
    if(i==M_i)
        subplot(2,2,1)
        legend(LegendNote);
        subplot(2,2,2)
        legend(LegendNote);
        subplot(2,2,3)
        legend(LegendNote);
        if(options.ChirpZ==1)
            subplot(2,2,4)
            legend(LegendNote);
        end
    end
end




